import { Notice, Plugin } from "obsidian";
import * as localforage from "localforage";
import TestWorker from "sqlite3.worker";
import { sqlite3Worker1Promiser } from "@sqlite.org/sqlite-wasm";

// Remember to rename these classes and interfaces!

interface Benchmark {
	time: number;
	name: string;
}

export default class MyPlugin extends Plugin {
	public persister: LocalForage;
	settings: any;
	private worker: Worker;
	private promiser: (...args: any[]) => any;
	public benches: Benchmark[] = [];
	private dbId: string;
	async onload() {
		await this.loadSettings();
		this.benches.push(
			await this.time("sqlite-init", async () => await this.initSqlite())
		);
		await this.sqlite();
		this.benches.push(
			await this.time("idb-init", async () => await this.initIDB())
		);
		await this.idb();
		console.table(this.benches)
	}

	onunload() {
		this.worker.terminate();
	}

	async loadSettings() {
		this.settings = Object.assign({}, await this.loadData());
	}

	async saveSettings() {
		await this.saveData(this.settings);
	}

	printNotice(...args: any) {
		console.log(...args);
		new Notice("~LOG~", args.join("\n"));
	}
	locateFile(path: string, prefix: string): string {
		return this.app.vault.adapter.getResourcePath(
			`${this.manifest.dir}/${path}`
		);
	}

	async time(benchName: string, func: () => Promise<void>): Promise<Benchmark> {
		const start = performance.now();
		await func();
		const end = performance.now();
		return {
			name: benchName,
			time: end - start,
		};
	}

	async initIDB() {
		this.persister = localforage.createInstance({
			name: "sqlite-bench",
			driver: localforage.INDEXEDDB,
			description: "we do a little benchmarking",
		});
	}

	async initSqlite() {
		this.worker = new TestWorker();
		let w = this.worker;
		this.worker.postMessage({
			type: "init",
			path: this.locateFile("sqlite3.wasm", ""),
		});
		this.promiser = await new Promise<typeof this.promiser>((resolve) => {
			const _promiser = sqlite3Worker1Promiser({
				print: this.printNotice,
				printErr: console.error,
				locateFile: this.locateFile.bind(this),
				onready: () => {
					resolve(_promiser);
				},
				worker: () => w,
			});
		});
		(window as any).sqlite3Promiser = this.promiser;
	}

	async doInsert(each: (s: number) => Promise<void>) {
		for (let i = 0; i < 50; i++) {
			await each(i);
		}
	}

	async sqlite() {
		this.benches.push(
			await this.time("sqlite-open", async () => {
				let openRes = await this.promiser("open", {
					filename: "/managed-sah/test.sqlite3",
					vfs: "opfs-sahpool",
				});
				const { dbId } = openRes;
				this.dbId = dbId;
				console.log(openRes);
			})
		);
		try {
			this.benches.push(await this.time("sqlite-create-table", async () => {
			console.log("Creating a table...");
				let res = await this.promiser("exec", {
					dbId: this.dbId,
					sql: "DROP TABLE IF EXISTS abcdef; CREATE TABLE IF NOT EXISTS abcdef(a,b,c)",
					returnValue: "resultRows",
				});
				console.log("Result: ", res);
			}));

			this.benches.push(
				await this.time("sqlite-insert-many", async () => {
					console.log("Insert some data using exec()...");
					await this.doInsert(async (i) => {
						let innerRes = await this.promiser("exec", {
							dbId: this.dbId,
							sql: "INSERT INTO abcdef(a,b,c) VALUES (?,?,69)",
							bind: [i, i * 2],
							returnValue: "resultRows",
							rowMode: "object",
						});
						// console.log(innerRes);
					});
				})
			);
			this.benches.push(
				await this.time("sqlite-query-many", async () => {
					console.log("Query data with exec()...");
					let rowRes = await this.promiser("exec", {
						dbId: this.dbId,
						sql: "SELECT * FROM abcdef ORDER BY a LIMIT 50",
						returnValue: "resultRows",
						rowMode: "object",
					});
					console.log(rowRes);
				})
			);
		} finally {
			await this.promiser("close", { dbId: this.dbId });
		}
	}
	async idb() {
		this.benches.push(await this.time("idb-insert-many", async() => {
			await this.doInsert(async (s) => {
				await this.persister.setItem("i/" +s, "brian tatler fucked and abused sean harris\n".repeat(Math.floor(s * 1.1)))
			})
		}))
		this.benches.push(await this.time("idb-query-many", async() => {
			let q: any[] = [];
			await this.doInsert(async (s) => {
				q.push(await this.persister.getItem("i/" + s))
			})
			console.log("qlen", q.length)
		}))
	}
}
